from openUtils import OpenUtils

if __name__ == '__main__':
    myOpenUtils = OpenUtils()
    myOpenUtils.initParams(
        "/Volumes/pioneer/gdal_Demo/cldas_/Z_NAFP_C_BABJ_20180701091809_P_CLDAS_RT_ASI_0P0625_HOR-PRS-2018070109.nc",
        file_type="nc",
        out_file="./abc.nc",
        export_type="nc",
        data_type='float32',
        values_strs=["lats", "lons", "abc"],
        nc_values=["LAT", "LON", "PAIR"])



__version__ = '$Id: umOpener.py 27349 2014-05-16 18:58:51Z rouault $'
type_list = ('nc', 'grib2', 'img', 'GeoTiff')
export_list = ('nc', 'grib2', 'img', 'GeoTiff')
datatype_list = ('int8', 'int16', 'float32', 'float64', 'float128')


class OpenUtils():
    def __init__(self):
        pass

    def initParams(self, inputFile, **kwargs):
        arguments = []
        for kwarg_key in kwargs.keys():
            arguments.append("--%s" % kwarg_key)
            arguments.append(kwargs[kwarg_key])
        self.inputFile = inputFile
        self.optparse_init()
        (self.options, self.args) = self.parser.parse_args(args=arguments)
        self.openFile()
        self.process()

    # -------------------------------------------------------------------------

    def optparse_init(self):
        """Prepare the option parser for input (argv)"""
        from optparse import OptionParser, OptionGroup
        usage = 'Usage: %prog [options] input_file(s) [output]'
        p = OptionParser(usage, version='%prog ' + __version__)
        p.add_option(
            '-t',
            '--file_type',
            dest='intPutType',
            type='choice',
            choices=type_list,
            help="Tile cutting profile (%s) - default 'mercator' (Google Maps compatible)"
                 % ','.join(type_list),
        )
        p.add_option(
            '-o',
            '--out_file',
            dest='outFile',
            help="out_file", )
        p.add_option(
            '-e',
            '--export_type',
            dest='exportType',
            type='choice',
            choices=export_list,
            help='export file type(%s)' % ",".join(export_list)
        )
        p.add_option(
            '-d',
            '--data_type',
            dest='dataType',
            type='choice',
            choices=datatype_list,
            help='output file data type'
        )
        p.add_option(
            '-n',
            '--nc_values',
            dest='ncValues',
            help='intput every band name',
        )
        p.add_option(
            '-v',
            '--values_strs',
            dest='valueStrs',
            help='output every band name',
        )
        p.set_defaults(
            export_type='GTiff',
            data_type="float32"
        )

        self.parser = p

    # =================================================================================================
    def openFile(self):
        if (self.options.intPutType == "nc"):
            import netcdf4reader
            nc_data, nc_attrs = netcdf4reader.read(self.inputFile, self.options.ncValues, self.options.dataType)
            self.lats = nc_data[0]
            self.lons = nc_data[1]
            self.nc_data = nc_data
            self.nc_attrs = nc_attrs
            # self.noData = []
            # for value in self.options.ncValues:
            #     if (value in nc_attrs.keys()):
            #         if ("_FillValue" in nc_attrs[value].keys()):
            #             self.noData.append(nc_attrs[value]["_FillValue"])
            #         else:
            #             self.noData.append(None)

            return nc_data

    # =================================================================================================
    def wirteNetcdfFile(self):
        if (self.options.exportType == "nc"):
            print "create netcdf4"
            import netcdf4reader
            netcdf4reader.wirte(self.options.outFile, self.nc_data, self.options.valueStrs, self.nc_attrs,
                                self.options.dataType)

    # ==================================================================================================
    def process(self):
        if (self.options.exportType == "nc"):
            self.wirteNetcdfFile()



# -*- coding:utf-8 -*-
from netCDF4 import Dataset
import os
import sys
import numpy as np


def wirte(output_file, datas, values_strs, nc_attrs, data_type):
    nc_dst = Dataset(output_file, 'w', format='NETCDF4')
    lat_nsize = datas[0].__len__()  # 维度的范围
    lon_nsize = datas[1].__len__()  # 经度的范围
    x = values_strs[1]
    y = values_strs[0]
    nc_dst.createDimension(y, lat_nsize);
    nc_dst.createDimension(x, lon_nsize);
    nctype = switType(data_type)
    if (datas.__len__() == values_strs.__len__() == nc_attrs.__len__()):
        if "_FillValue" in nc_attrs[0].keys():
            y_miss_value = nc_attrs[0]["_FillValue"]
            var_value = nc_dst.createVariable(values_strs[0], nctype, (y), fill_value=y_miss_value)
            var_value.setncatts(nc_attrs[0].pop("_FillValue"))
        else:
            var_value = nc_dst.createVariable(values_strs[0], nctype, (y))
            var_value.setncatts(nc_attrs[0])

        nc_dst.variables[values_strs[0]][:] = datas[0]

        if "_FillValue" in nc_attrs[1].keys():
            x_miss_value = nc_attrs[1]["_FillValue"]
            var_value = nc_dst.createVariable(values_strs[1], nctype, (x), fill_value=x_miss_value)
            var_value.setncatts(nc_attrs[1].pop("_FillValue"))
        else:
            var_value = nc_dst.createVariable(values_strs[1], nctype, (x))
            var_value.setncatts(nc_attrs[1])
        nc_dst.variables[values_strs[1]][:] = datas[1]

        for values_str, nc_attr, data in zip(values_strs[2:], nc_attrs[2:], datas[2:]):
            if "_FillValue" in nc_attr.keys():
                miss_value = nc_attr['_FillValue']
                var_value = nc_dst.createVariable(values_str, nctype, (y, x), fill_value=miss_value)
                nc_attr.pop("_FillValue")
            else:
                var_value = nc_dst.createVariable(values_str, nctype, (y, x))
            var_value.setncatts(nc_attr)
            nc_dst.variables[values_str][:] = data
    del nc_dst


# ===================================================================


def read(input_file, values, dataType):
    if (os.path.exists(input_file)):
        nc_ds = Dataset(input_file)
    else:
        print "%s is not exist" % input_file
        sys.exit()
    nc_data = []
    nc_attrs = []
    for value in values:
        if value in nc_ds.variables.keys():
            data = nc_ds.variables[value][:]
            data = np.array(data, dtype=dataType)
            attrs = {}
            for attr in nc_ds.variables[value].ncattrs():
                attrs[attr] = getattr(nc_ds.variables[value], attr)
            nc_attrs.append(attrs)
            nc_data.append(data)
        else:
            print "%s value error " % value
            sys.exit()
    nc_data = np.array(nc_data)
    del nc_ds
    return nc_data, nc_attrs


def switType(data_type):
    if (data_type == "float32" or data_type == "float64"):
        nctype = "f"
    elif (data_type == "int16" or data_type == "int8"):
        nctype = "i"
    return nctype
